/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2011-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2011-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

package scalasca.cubex.cube.datalayout.data;
import scalasca.cubex.cube.datalayout.data.value.*;
import scalasca.cubex.cube.errors.*;
import scalasca.cubex.cube.services.transformation.*;
import java.lang.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
/*
 * Class , delivers file structure of CubeX
 * 
 */
public class CompressedDataRows extends DataRows
{
	public CompressedDataRows(byte[] data, Value value, Endianess endianess)  throws BadDataException
	{
		super(data, value, endianess);
	};

	public void print()
		{
			System.out.println("======== Compressed rows of data ============");
		}

	protected void loadData(byte[] _data, Value _value, Endianess _endianess)  throws BadDataException
	{
		byte[] uncompressed_data = decompress(_data, _endianess);
		super.loadData(uncompressed_data, _value, _endianess);
	}


	protected byte[] decompress(byte[] _compressed_data, Endianess _endianess ) throws BadDataException
	{
		try
		{
			DataInputStream in = new DataInputStream( new ByteArrayInputStream(_compressed_data));
			zHeader zheader = new zHeader(in, _endianess);
			ByteArrayOutputStream b_out = new ByteArrayOutputStream();

			for (zTriplet tiplet : zheader.getTriplets())
			{
				long to_allocate = tiplet.getCompressedSize();
				if (to_allocate == 0)
					continue;
				byte[] _tmp = new byte[(int)(to_allocate )];
				in.read(_tmp);

				ByteArrayInputStream b_in = new ByteArrayInputStream(_tmp);
				InflaterInputStream z_in = new InflaterInputStream(b_in);
				byte[] __tmp = new byte[5*1024*1024];
				while (z_in.available() != 0)
				{
					int read = z_in.read(__tmp, 0, 5*1024*1024);
					if (read == -1)
						break;
					b_out.write(__tmp, 0, read);
				}
				_tmp = null;
				__tmp = null;
			}
			return b_out.toByteArray();
		}catch(IOException e)
		{
			throw new BadDataException("Error in loading header for compressed data: " + e.toString());
		}

	}


}
