/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2011-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2011-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

package scalasca.cubex.cube.datalayout.data.value;
import scalasca.cubex.cube.errors.*;
import scalasca.cubex.cube.services.transformation.*;

import java.lang.*;
import java.util.*;
import java.io.*;
/*
 * Class , delivers file structure of CubeX
 * 
 */
public class CubeUint32 extends Value
{
	protected long value; // we work with long, buut store in stream only lowest 4 bytes

	public CubeUint32() {};
	@SuppressWarnings("unchecked")
	public CubeUint32( int _v) { value = (long)_v; };

	public String getName()
		{
			return "UINT32";
		}


	public String toString()
	{
		return Long.valueOf(value).toString();
	}

	public int size() { return 4; } ;

	public void correctEndianess(Endianess _end, DataInputStream in, DataOutputStream out)  throws IOException
		{
			int _in  =  in.readInt();
			int __in = _end.applyOn(_in);
			out.writeInt(__in);
		}
	public Value clone()
	{
		return new CubeUint32();
	}
	public Value copy()
	{
		return new CubeUint32((int)value);
	}
	public Value clone(DataInputStream in)  throws IOException
	{
		return new CubeUint32(in.readInt());
	}

 // ------------------- setXXX methods ------------------
	public  void setValue(String v) // initialization from String
	{
		value = Long.parseLong(v);
	}
	@SuppressWarnings("unchecked")
	public  void setValue(byte v) // initialization from byte
	{
		value = (long)v;
	}

	public  void setValue(short v) // initialization from short
	{
		value = (long)v;
	}
	public  void setValue(int v) // initialization from integer
	{
		value = (long)v;
	}
	public  void setValue(long v) // initialization from long
	{
		value = v;
	}
	public  void setValue(double v) // initialization from double
	{
		value = (long)v;
	}


 // ------------------- getXXX methods ------------------
	public double getDouble()
	{
		return Long.valueOf(value).doubleValue();
	}

 // -------------------- lagebra -------------------------
	public void addValue(Value v)  // performs operation "+"
	{
		CubeUint32 _v = (CubeUint32)v;
		value += _v.value;
	}

	public void subtractValue(Value v)  // performs operation "-", if defined
	{
		CubeUint32 _v = (CubeUint32)v;
		value -= _v.value;
	}

}
