/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2011-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2011-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

package scalasca.cubex.cubexlayout;
import scalasca.cubex.tarviewer.*;
import org.xeustechnologies.jtar.*;
import java.io.FileInputStream;
import java.lang.*;
import java.util.*;
import scalasca.cubex.cube.errors.*;
import java.io.*;

/**
 * Provides for a cube parser information about the place where to look for an anchor and metric data files if it is cubex report. 
 * Besides of that  reads content of that file and returns as an array byte[]. Cube uses that array and does not read in file.
* It perform also check for the format.
*/
public class CubeTarLayout extends CubeLayout
{

	@SuppressWarnings("unchecked")
	public CubeTarLayout(String filename)
	{
		super(filename);
		try
		{
			BTarInputStream tis = new BTarInputStream(new FileInputStream(filename));
			BTarEntry te;
			while((te = tis.next()) != null)
			{
				if (te.getName().compareTo("anchor.xml") == 0)
				{
					AnchorShift = te.getOffset();
					AnchorSize = te.getSize();
				}
				else
				if (te.getName().endsWith(".data") )
				{
					String id = te.getName().substring(0, te.getName().indexOf(".data"));
					Long _id = new Long(id);
					HashMap _pair = new HashMap();
					_pair.put("size",   new Long(te.getSize()));
					_pair.put("offset", new Long(te.getOffset()));
					metricDataPlaces.put( _id,  _pair );
				}
				else
				if (te.getName().endsWith(".index") )
				{
					String id = te.getName().substring(0, te.getName().indexOf(".index"));
					Long _id = new Long(id);
					HashMap _pair = new HashMap();
					_pair.put("size",   new Long(te.getSize()));
					_pair.put("offset", new Long(te.getOffset()));
					metricIndexPlaces.put(_id, _pair );
				}
				else				{
					String name = te.getName();
					HashMap _pair = new HashMap();
					_pair.put("size",   new Long(te.getSize()));
					_pair.put("offset", new Long(te.getOffset()));
					miscDataPlaces.put(name, _pair );
				}

			}
			if (AnchorSize == -1 && AnchorShift == -1 )
				throw new BadCubeReportLayoutException("Not found anchor in cubex file " + filename);
		}catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}
	}



	public long getAnchorSize()
	{
		return AnchorSize;
	}

	public long getAnchorShift()
	{
		return AnchorShift;
	}



	public long getMetricDataSize(int id) throws MissingMetricInCubeReport
	{
		Map<String, Long>  _met_pair = metricDataPlaces.get(new Long(id));

		if (_met_pair == null)
			throw new MissingMetricInCubeReport("Missing "+id+".data file in cube report");

		return _met_pair.get("size").longValue();
	}

	public long getMetricDataShift(int id) throws MissingMetricInCubeReport
	{
		Map<String, Long>  _met_pair = metricDataPlaces.get(new Long(id));
		if (_met_pair == null)
			throw new MissingMetricInCubeReport("Missing "+id+".data file in cube report");
		return _met_pair.get("offset").longValue();
	}


	public long getMetricIndexSize(int id) throws MissingMetricInCubeReport
	{
		Map<String, Long>  _met_pair = metricIndexPlaces.get(new Long(id));
		if (_met_pair == null)
			throw new MissingMetricInCubeReport("Missing "+id+".index file in cube report");
		return _met_pair.get("size").longValue();
	}

	public long getMetricIndexShift(int id) throws MissingMetricInCubeReport
	{
		Map<String, Long>  _met_pair = metricIndexPlaces.get(new Long(id));
		if (_met_pair == null)
			throw new MissingMetricInCubeReport("Missing "+id+".index file in cube report");
		return _met_pair.get("offset").longValue();
	}

	



	public long getMiscDataSize(String name) throws MissingMiscInCubeReport
	{
		Map<String, Long>  _misc_pair = miscDataPlaces.get(name);

		if (_misc_pair == null)
			throw new MissingMiscInCubeReport("Missing "+name+" file in cube report");

		return _misc_pair.get("size").longValue();
	}

	public long getMiscDataShift(String name) throws MissingMiscInCubeReport
	{
		Map<String, Long>  _misc_pair = miscDataPlaces.get(name);
		if (_misc_pair == null)
			throw new MissingMiscInCubeReport("Missing "+name+" file in cube report");
		return _misc_pair.get("offset").longValue();
	}


	

	public void print()
	{
		System.out.println("Anchor size:      " + AnchorSize);
		System.out.println("Anchor offset:   " + AnchorShift);

		System.out.println("===== Data =====");
		for (Long key : metricDataPlaces.keySet())
		{
			Map<String, Long> _met_pair = metricDataPlaces.get(key);
			System.out.println(key+".data -> (" + _met_pair.get("size") + ", " + _met_pair.get("offset") + ")");
		}
		System.out.println("===== Indecies =====");
		for (Long key : metricIndexPlaces.keySet())
		{
			Map<String, Long> _met_pair = metricIndexPlaces.get(key);
			System.out.println(key+".index -> (" + _met_pair.get("size") + ", " + _met_pair.get("offset") + ")");
		}

	}


	static boolean testLayout(String _filename)
	{
		try
		{
			TarInputStream tis = new TarInputStream(new FileInputStream(_filename));
			if (tis.getNextEntry().getHeader().magic.substring(0, "ustar".length()).equalsIgnoreCase("ustar"))
			{
				return true;
			}else
			{
				return false;
			}

		}catch (IOException e)
		{
			return false;
		}
	}









	public byte[] getAnchorBytes() throws IOException
	{
		try{
			FileInputStream  fis = new FileInputStream(filename);
			fis.skip(AnchorShift);

			byte anchor[] = new byte[(int)AnchorSize];

			fis.read(anchor);
			fis.close();
			return anchor;
		}catch (FileNotFoundException e)
		{
			System.out.println("File " + filename + " not found.");
			return new byte[0];
		}
	}


	public byte[] getMetricDataBytes(int id) throws IOException
	{
		File file= new File(filename);
		byte metric_data[] = null;
		try 
		{
		  long metric_data_size = this.getMetricDataSize(id);
		  long metric_data_offset = this	.getMetricDataShift(id);
		  FileInputStream _fdata = new FileInputStream(file);
		  _fdata.skip(metric_data_offset);
		  metric_data = new byte[(int)metric_data_size];
		  _fdata.read(metric_data);
		  _fdata.close();
		  }
		  catch( MissingMetricInCubeReport e){}
		return metric_data;
	}

	
	
	public byte[] getMetricIndexBytes(int id) throws IOException
	{
		File file= new File(filename);
		byte metric_index[] = null;
		try 
		{
		  long metric_index_size = this.getMetricIndexSize(id);
		  long metric_index_offset = this.getMetricIndexShift(id);
		  FileInputStream _findex = new FileInputStream(file);
		  _findex.skip(metric_index_offset);
		  metric_index = new byte[(int)metric_index_size];
		  _findex.read(metric_index);
		  _findex.close();
		}
		catch( MissingMetricInCubeReport e){}
		return metric_index;
	}

	public byte[] getMiscDataBytes(String name) throws IOException, MissingMiscInCubeReport
	{
		File file= new File(filename);
		long misc_data_size = this.getMiscDataSize(name);
		long misc_data_offset = this	.getMiscDataShift(name);
		FileInputStream _fdata = new FileInputStream(file);
		_fdata.skip(misc_data_offset);
		byte misc_data[] = new byte[(int)misc_data_size];
		_fdata.read(misc_data);
		_fdata.close();
		return misc_data;
	}










}
